﻿using System.Windows.Forms;

using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;

namespace TestProject
{
    /// <summary>
    /// 포커스 행 헬퍼
    /// </summary>
    public class FocusRowHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 그리드 뷰
        /// </summary>
        private GridView gridView = null;

        /// <summary>
        /// 이전 포커스 사각형 그리기 스타일
        /// </summary>
        private DrawFocusRectStyle previousDrawFocusRectStyle;

        /// <summary>
        /// 포커스 행 하이라이트 여부
        /// </summary>
        private bool focusRowVisible = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 첫번째 클릭시 포커스 행 표시 여부 - ShowFocusRowOnFirstClick

        /// <summary>
        /// 첫번째 클릭시 포커스 행 표시 여부
        /// </summary>
        public bool ShowFocusRowOnFirstClick { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FocusRowHelper(gridView)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="gridView">그리드 뷰</param>
        public FocusRowHelper(GridView gridView)
        {
            this.gridView = gridView;

            ShowFocusRowOnFirstClick = true;

            Activate();

            SetFocusRowVisible(false);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 활성화 시키기 - Activate()

        /// <summary>
        /// 활성화 시키기
        /// </summary>
        public void Activate()
        {
            this.gridView.CustomDrawRowIndicator += GridView_CustomDrawRowIndicator;
            this.gridView.MouseDown              += GridView_MouseDown;
        }

        #endregion
        #region 비활성화 시키기 - Deactivate()

        /// <summary>
        /// 비활성화 시키기
        /// </summary>
        public void Deactivate()
        {
            SetFocusRowVisible(true);

            this.gridView.CustomDrawRowIndicator -= GridView_CustomDrawRowIndicator;
            this.gridView.MouseDown              -= GridView_MouseDown;
        }

        #endregion

        #region 포커스 행 표시 여부 설정하기 - SetFocusRowVisible(visible)

        /// <summary>
        /// 포커스 행 표시 여부 설정하기
        /// </summary>
        /// <param name="visible">표시 여부</param>
        public void SetFocusRowVisible(bool visible)
        {
            this.focusRowVisible = visible;

            if(!visible)
            {
                this.previousDrawFocusRectStyle = this.gridView.FocusRectStyle;
            }

            this.gridView.BeginUpdate();

            this.gridView.FocusRectStyle = visible ? this.previousDrawFocusRectStyle : DrawFocusRectStyle.None;

            this.gridView.OptionsSelection.EnableAppearanceFocusedCell   = this.focusRowVisible;
            this.gridView.OptionsSelection.EnableAppearanceFocusedRow    = this.focusRowVisible;
            this.gridView.OptionsSelection.EnableAppearanceHideSelection = this.focusRowVisible;

            this.gridView.EndUpdate();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 행 지시자 커스텀 그리기 - GridView_CustomDrawRowIndicator(sender, e)

        /// <summary>
        /// 그리드 뷰 행 지시자 커스텀 그리기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void GridView_CustomDrawRowIndicator(object sender, RowIndicatorCustomDrawEventArgs e)
        {
            if(!this.focusRowVisible)
            {
                e.Info.ImageIndex = -1;
            }
        }

        #endregion
        #region 그리드 뷰 마우스 DOWN 처리하기 - GridView_MouseDown(sender, e)

        /// <summary>
        /// 그리드 뷰 마우스 DOWN 처리하기 
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void GridView_MouseDown(object sender, MouseEventArgs e)
        {
            if(ShowFocusRowOnFirstClick)
            {
                GridHitInfo gridHitInfo = this.gridView.CalcHitInfo(e.Location);

                if(gridHitInfo.InRow)
                {
                    SetFocusRowVisible(true);
                }
            }
        }

        #endregion
    }
}